/***
*
*	Copyright (c) 1999, Valve LLC. All rights reserved.
*	
*	This product contains software technology licensed from Id 
*	Software, Inc. ("Id Technology").  Id Technology (c) 1996 Id Software, Inc. 
*	All Rights Reserved.
*
*   Use, distribution, and modification of this source code and/or resulting
*   object code is restricted to non-commercial enhancements to products from
*   Valve LLC.  All other use, distribution, or modification is prohibited
*   without written permission from Valve LLC.
*
****/

/*------------------------------------------------------------------------------
*
*	Half-Life JailBreak Revision Log
*
*	$History: scoreboard.cpp $
*       
*       *****************  Version 5  *****************
*       User: Tim Smith    Date: 4/27/99    Time: 10:08a
*       Updated in $/HLJB/cl_dll
*       Completed team based scoring system
*       
*       *****************  Version 4  *****************
*       User: Tim Smith    Date: 4/26/99    Time: 9:13p
*       Updated in $/HLJB/cl_dll
*       1) Cleaned up observer team display
*       2) Added a new centered console print routine to be used later
*       
*       *****************  Version 3  *****************
*       User: Tim Smith    Date: 4/17/99    Time: 11:48a
*       Updated in $/HLJB/cl_dll
*       Added support for the trigger_armory entity
*       
*       *****************  Version 2  *****************
*       User: Tim Smith    Date: 4/15/99    Time: 9:55a
*       Updated in $/HLJB/cl_dll
*       Initial version of the HUD completed
*       
*       *****************  Version 1  *****************
*       User: Tim Smith    Date: 4/14/99    Time: 12:04p
*       Created in $/HLJB/cl_dll
*       
*-----------------------------------------------------------------------------*/

//JAIL+++
// The score board has been totally redone 
//JAIL---

//
// Scoreboard.cpp
//
// implementation of CHudScoreboard class
//

#include "hud.h"
#include "util.h"
#include "parsemsg.h"

#include <string.h>
#include <stdio.h>

DECLARE_COMMAND( m_Scoreboard, ShowScores );
DECLARE_COMMAND( m_Scoreboard, HideScores );

DECLARE_MESSAGE( m_Scoreboard, ScoreInfo );
DECLARE_MESSAGE( m_Scoreboard, JTeamScore );

DECLARE_MESSAGE( m_Scoreboard, JTeamInfo );
DECLARE_MESSAGE( m_Scoreboard, JTeamList );
DECLARE_MESSAGE( m_Scoreboard, JInJail );
DECLARE_MESSAGE( m_Scoreboard, JInArmory );

int CHudScoreboard :: Init( void )
{
	gHUD.AddHudElem( this );

	// Hook messages & commands here
	HOOK_COMMAND( "+showscores", ShowScores );
	HOOK_COMMAND( "-showscores", HideScores );

	HOOK_MESSAGE( ScoreInfo );
	HOOK_MESSAGE( JTeamScore );
	HOOK_MESSAGE( JTeamInfo );
	HOOK_MESSAGE( JTeamList );
	HOOK_MESSAGE( JInJail );
	HOOK_MESSAGE( JInArmory );

	InitHUDData();

	return 1;
}


int CHudScoreboard :: VidInit( void )
{
	// Load sprites here

	return 1;
}

void CHudScoreboard :: InitHUDData( void )
{
	memset( m_PlayerExtraInfo, 0, sizeof m_PlayerExtraInfo );
	m_iLastKilledBy = 0;
	m_fLastKillTime = 0;
	m_iPlayerNum = 0;
	m_iNumTeams = 0;
	m_fTeamScore = FALSE;
	memset( m_TeamInfo, 0, sizeof m_TeamInfo );

	m_iFlags &= ~HUD_ACTIVE;  // starts out inactive

	m_iFlags |= HUD_INTERMISSION; // is always drawn during an intermission
}

/* The scoreboard
We have a minimum width of 1-320 - we could have the field widths scale with it?
*/

// X positions
// relative to the side of the scoreboard
#define NAME_RANGE_MIN  20
#define NAME_RANGE_MAX  145
#define EXEC_RANGE_MIN 65
#define EXEC_RANGE_MAX 115
#define DIVIDER_POS2	125
#define KILLS_RANGE_MIN 130
#define KILLS_RANGE_MAX 170
#define DIVIDER_POS		180
#define DEATHS_RANGE_MIN  185
#define DEATHS_RANGE_MAX  210
#define PING_RANGE_MIN	245
#define PING_RANGE_MAX	295

#define SCOREBOARD_WIDTH 320
		
void CHudScoreboard :: TallyScores( )
{
	GetAllPlayersInfo();

	// clear out team scores
	for ( int i = 1; i <= m_iNumTeams; i++ )
	{
		m_TeamInfo[i].frags = m_TeamInfo[i].deaths = 0;
		m_TeamInfo[i].ping = m_TeamInfo[i].packetloss = 0;
		m_TeamInfo[i].injail = 0;
	}

	// recalc the team scores, then draw them
	for ( i = 1; i < MAX_PLAYERS; i++ )
	{
		if ( m_PlayerInfoList[i].name == NULL )
			continue; // empty player slot, skip

		if ( m_PlayerExtraInfo[i].teamindex == 0 )
			continue; // skip over players who are not in a team

		int j = m_PlayerExtraInfo [i] .teamindex;

		if ( j > m_iNumTeams )  // player is not in a team, skip to the next guy
			continue;

		m_TeamInfo[j].frags += m_PlayerExtraInfo[i].frags;
		m_TeamInfo[j].deaths += m_PlayerExtraInfo[i].deaths;

		m_TeamInfo[j].ping += m_PlayerInfoList[i].ping;
		m_TeamInfo[j].packetloss += m_PlayerInfoList[i].packetloss;
		if (m_PlayerExtraInfo [i] .injail)
			m_TeamInfo [j] .injail++;
	}

	// find team ping/packetloss averages
	for ( i = 1; i <= m_iNumTeams; i++ )
	{
		if ( m_TeamInfo[i].players > 0 )
		{
			m_TeamInfo[i].ping /= m_TeamInfo[i].players;  // use the average ping of all the players in the team as the teams ping
			m_TeamInfo[i].packetloss /= m_TeamInfo[i].players;  // use the average ping of all the players in the team as the teams ping
		}
	}

	// if team score is not overridden, copy frags to executions

	for ( i = 1; i <= m_iNumTeams; i++ )
	{
		if ( !m_TeamInfo[i].scores_overriden )
		{
			m_TeamInfo [i] .executions = m_TeamInfo [i] .frags;
		}
	}

}

// Y positions
#define ROW_GAP  13
#define ROW_RANGE_MIN 15
#define ROW_RANGE_MAX ( ScreenHeight - 50 )

int CHudScoreboard :: Draw( float fTime )
{
	if ( !m_iShowscoresHeld && gHUD.m_Health.m_iHealth > 0 && !gHUD.m_iIntermission )
		return 1;

	//GetAllPlayersInfo(); (done in tally)

	// just sort the list on the fly
	// list is sorted first by frags, then by deaths
	float list_slot = 0;
	int xpos_rel = (ScreenWidth - SCOREBOARD_WIDTH) / 2;

	// print the heading line
	int ypos = ROW_RANGE_MIN + (list_slot * ROW_GAP);
	int	xpos = NAME_RANGE_MIN + xpos_rel;

	if ( !gHUD.m_Teamplay ) 
		gHUD.DrawHudString( xpos, ypos, NAME_RANGE_MAX + xpos_rel, "Player", 255, 140, 0 );
	else
		gHUD.DrawHudString( xpos, ypos, NAME_RANGE_MAX + xpos_rel, "Teams", 255, 140, 0 );

	if (m_fTeamScore)
	{
		gHUD.DrawHudStringReverse( EXEC_RANGE_MAX + xpos_rel, ypos, 0, "execs", 255, 140, 0 );
		gHUD.DrawHudString( DIVIDER_POS2 + xpos_rel, ypos, ScreenWidth, "/", 255, 140, 0 );
	}

	gHUD.DrawHudStringReverse( KILLS_RANGE_MAX + xpos_rel, ypos, 0, "kills", 255, 140, 0 );
	gHUD.DrawHudString( DIVIDER_POS + xpos_rel, ypos, ScreenWidth, "/", 255, 140, 0 );
	gHUD.DrawHudString( DEATHS_RANGE_MIN + xpos_rel + 5, ypos, ScreenWidth, "deaths", 255, 140, 0 );
	gHUD.DrawHudString( PING_RANGE_MAX + xpos_rel - 35, ypos, ScreenWidth, "latency", 255, 140, 0 );

	list_slot += 1.2;
	ypos = ROW_RANGE_MIN + (list_slot * ROW_GAP);
	xpos = NAME_RANGE_MIN + xpos_rel;
	FillRGBA( xpos - 5, ypos, PING_RANGE_MAX - 5, 1, 255, 140, 0, 255);  // draw the seperator line
	
	list_slot += 0.8;

	if ( !gHUD.m_Teamplay )
	{
		// it's not teamplay,  so just draw a simple player list
		DrawPlayers( xpos_rel, list_slot );
		return 1;
	}

	// clear the drawn flags
	for ( int i = 1; i <= m_iNumTeams; i++ )
	{
		m_TeamInfo[i].already_drawn = FALSE;
	}

	// Draw the teams
	while ( 1 )
	{
		int highest_exec = -99999;
		int highest_frags = -99999; int lowest_deaths = 99999;
		int best_team = 0;

		for ( i = 1; i <= m_iNumTeams - 1; i++ )
		{
			if (m_TeamInfo[i].players < 0 )
				continue;

			if (m_TeamInfo [i] .already_drawn) 
				continue;

			if (m_TeamInfo [i] .executions < highest_exec)
				continue;

			if (m_TeamInfo [i] .executions == highest_exec)
			{
				if (m_TeamInfo [i] .frags < highest_frags)
					continue;

				if (m_TeamInfo [i] .frags == highest_frags)
				{
					if (m_TeamInfo [i] .deaths >= lowest_deaths)
						continue;
				}
			}

			best_team = i;
			lowest_deaths = m_TeamInfo[i].deaths;
			highest_frags = m_TeamInfo[i].frags;
			highest_exec = m_TeamInfo [i] .executions;
		}

		// draw the best team on the scoreboard
		if ( !best_team )
			break;

		// draw out the best team
		team_info_t *team_info = &m_TeamInfo[best_team];

		ypos = ROW_RANGE_MIN + (list_slot * ROW_GAP);

		// check we haven't drawn too far down
		if ( ypos > ROW_RANGE_MAX )  // don't draw to close to the lower border
			break;

		xpos = NAME_RANGE_MIN + xpos_rel;
		int r = 255, g = 225, b = 55; // draw the stuff kinda yellowish
		
		switch (best_team)
		{
			case 1:
				FillRGBA( NAME_RANGE_MIN + xpos_rel - 5, ypos, 
					PING_RANGE_MAX - 5, ROW_GAP, 255, 0, 0, 70 );
				break;
			case 2:
				FillRGBA( NAME_RANGE_MIN + xpos_rel - 5, ypos, 
					PING_RANGE_MAX - 5, ROW_GAP, 0, 0, 255, 70 );
				break;
			default:
			case 3: //ETS - if more than 2 teams are used, this must change
				//FillRGBA( NAME_RANGE_MIN + xpos_rel - 5, ypos, 
				//	PING_RANGE_MAX - 5, ROW_GAP, 0, 255, 255, 70 );
				break;
		}

		//if ( team_info->ownteam ) // if it is their team, draw the background different color
		//{
		//	// overlay the background in blue,  then draw the score text over it
		//	FillRGBA( NAME_RANGE_MIN + xpos_rel - 5, ypos, PING_RANGE_MAX - 5, ROW_GAP, 0, 0, 255, 70 );
		//}

		// draw their name (left to right)
		gHUD.DrawHudString( xpos, ypos, NAME_RANGE_MAX + xpos_rel, team_info->name, r, g, b );

		// draw executions (right to left)
		if (team_info ->scores_overriden)
		{
			xpos = EXEC_RANGE_MAX + xpos_rel;
			gHUD.DrawHudNumberString( xpos, ypos, EXEC_RANGE_MIN + xpos_rel, team_info->executions, r, g, b );
		}

		// draw kills (right to left)
		xpos = KILLS_RANGE_MAX + xpos_rel;
		gHUD.DrawHudNumberString( xpos, ypos, KILLS_RANGE_MIN + xpos_rel, team_info->frags, r, g, b );

		// draw divider
		xpos = DIVIDER_POS + xpos_rel;
		gHUD.DrawHudString( xpos, ypos, xpos + 20, "/", r, g, b );

		// draw deaths
		xpos = DEATHS_RANGE_MAX + xpos_rel;
		gHUD.DrawHudNumberString( xpos, ypos, DEATHS_RANGE_MIN + xpos_rel, team_info->deaths, r, g, b );

		// draw ping
		// draw ping & packetloss
		static char buf[64];
		sprintf( buf, "%d", team_info->ping );
		xpos = ((PING_RANGE_MAX - PING_RANGE_MIN) / 2) + PING_RANGE_MIN + xpos_rel + 25;
		UnpackRGB( r, g, b, RGB_YELLOWISH );
		gHUD.DrawHudStringReverse( xpos, ypos, xpos - 50, buf, r, g, b );

	/*  Packetloss removed on Kelly 'shipping nazi' Bailey's orders
		sprintf( buf, " %d", team_info->packetloss );
		gHUD.DrawHudString( xpos, ypos, xpos+50, buf, r, g, b );
	*/

		team_info->already_drawn = TRUE;  // set the already_drawn to be TRUE, so this team won't get drawn again
		list_slot++;

		// draw all the players that belong to this team, indented slightly
		list_slot = DrawPlayers( xpos_rel, list_slot, 10, best_team );
	}

	// draw all the players who are not in a team (observers)

	if (m_TeamInfo[m_iNumTeams].players > 0)
	{
		//
		// Draw header
		//

		ypos = ROW_RANGE_MIN + (list_slot * ROW_GAP);
		FillRGBA( NAME_RANGE_MIN + xpos_rel - 5, ypos, 
			PING_RANGE_MAX - 5, ROW_GAP, 255, 255, 255, 70 );
		xpos = NAME_RANGE_MIN + xpos_rel;
		int r = 255, g = 225, b = 55; // draw the stuff kinda yellowish
		gHUD.DrawHudString( xpos, ypos, NAME_RANGE_MAX + xpos_rel,
			"observers", r, g, b );
		list_slot += 1;
		DrawPlayers( xpos_rel, list_slot, 0, m_iNumTeams );
		list_slot += 1;
	}

#if 0
	if (gHUD .m_iIntermission)
	{
		int ypos = ROW_RANGE_MIN + (list_slot * ROW_GAP);
		gHUD .DrawHudStringCenter (xpos_rel, xpos_rel + SCOREBOARD_WIDTH,
			ypos, "This is a test", 255, 255, 255);

	}
#endif

	return 1;
}

// returns the ypos where it finishes drawing
int CHudScoreboard :: DrawPlayers( int xpos_rel, float list_slot, int nameoffset, int team )
{
	// draw the players, in order,  and restricted to team if set
	while ( 1 )
	{
		// Find the top ranking player
		int highest_frags = -99999;	int lowest_deaths = 99999;
		int best_player = 0;

		for ( int i = 1; i < MAX_PLAYERS; i++ )
		{
			if ( m_PlayerInfoList[i].name && m_PlayerExtraInfo[i].frags >= highest_frags )
			{
				if ( team == 0  || m_PlayerExtraInfo [i] .teamindex == team )  // make sure it is the specified team
				{
					extra_player_info_t *pl_info = &m_PlayerExtraInfo[i];
					if ( pl_info->frags > highest_frags || pl_info->deaths < lowest_deaths )
					{
						best_player = i;
						lowest_deaths = pl_info->deaths;
						highest_frags = pl_info->frags;
					}
				}
			}
		}

		if ( !best_player )
			break;

		// draw out the best player
		hud_player_info_t *pl_info = &m_PlayerInfoList[best_player];

		int ypos = ROW_RANGE_MIN + (list_slot * ROW_GAP);

		// check we haven't drawn too far down
		if ( ypos > ROW_RANGE_MAX )  // don't draw to close to the lower border
			break;

		int xpos = NAME_RANGE_MIN + xpos_rel;
		int r = 255, g = 255, b = 255;

		if (m_PlayerExtraInfo[best_player].injail)
		{
			FillRGBA( NAME_RANGE_MIN + xpos_rel - 5 + nameoffset, ypos, 
				PING_RANGE_MAX - 5 - nameoffset, ROW_GAP, 128, 128, 128, 70 );
		}
		else if (m_PlayerExtraInfo [best_player].inarmory)
		{
			FillRGBA( NAME_RANGE_MIN + xpos_rel - 5 + nameoffset, ypos, 
				PING_RANGE_MAX - 5 - nameoffset, ROW_GAP, 255, 225, 0, 70 );
		}

#if 0 //ETS no highlights
		if ( best_player == m_iLastKilledBy && m_fLastKillTime && m_fLastKillTime > gHUD.m_flTime )
		{
			if ( pl_info->thisplayer )
			{  // green is the suicide color? i wish this could do grey...
				FillRGBA( NAME_RANGE_MIN + xpos_rel - 5, ypos, PING_RANGE_MAX - 5, ROW_GAP, 80, 155, 0, 70 );
			}
			else
			{  // Highlight the killers name - overlay the background in red,  then draw the score text over it
				FillRGBA( NAME_RANGE_MIN + xpos_rel - 5, ypos, PING_RANGE_MAX - 5, ROW_GAP, 255, 0, 0, ((float)15 * (float)(m_fLastKillTime - gHUD.m_flTime)) );
			}
		}
		else if ( pl_info->thisplayer ) // if it is their name, draw it a different color
		{
			// overlay the background in blue,  then draw the score text over it
			FillRGBA( NAME_RANGE_MIN + xpos_rel - 5, ypos, PING_RANGE_MAX - 5, ROW_GAP, 0, 0, 255, 70 );
		}
#endif

		// draw their name (left to right)
		gHUD.DrawHudString( xpos + nameoffset, ypos, NAME_RANGE_MAX + xpos_rel, pl_info->name, r, g, b );

		// draw kills (right to left)
		xpos = KILLS_RANGE_MAX + xpos_rel;
		gHUD.DrawHudNumberString( xpos, ypos, KILLS_RANGE_MIN + xpos_rel, m_PlayerExtraInfo[best_player].frags, r, g, b );

		// draw divider
		xpos = DIVIDER_POS + xpos_rel;
		gHUD.DrawHudString( xpos, ypos, xpos + 20, "/", r, g, b );

		// draw deaths
		xpos = DEATHS_RANGE_MAX + xpos_rel;
		gHUD.DrawHudNumberString( xpos, ypos, DEATHS_RANGE_MIN + xpos_rel, m_PlayerExtraInfo[best_player].deaths, r, g, b );

		// draw ping & packetloss
		static char buf[64];
		sprintf( buf, "%d", m_PlayerInfoList[best_player].ping );
		xpos = ((PING_RANGE_MAX - PING_RANGE_MIN) / 2) + PING_RANGE_MIN + xpos_rel + 25;
		gHUD.DrawHudStringReverse( xpos, ypos, xpos - 50, buf, r, g, b );

	/*  Packetloss removed on Kelly 'shipping nazi' Bailey's orders
		if ( m_PlayerInfoList[best_player].packetloss >= 63 )
		{
			UnpackRGB( r, g, b, RGB_REDISH );
			sprintf( buf, " !!!!" );
		}
		else
		{
			sprintf( buf, " %d", m_PlayerInfoList[best_player].packetloss );
		}

		gHUD.DrawHudString( xpos, ypos, xpos+50, buf, r, g, b );
	*/

		pl_info->name = NULL;  // set the name to be NULL, so this client won't get drawn again
		list_slot++;
	}

	return list_slot;
}


void CHudScoreboard :: GetAllPlayersInfo( void )
{
	for ( int i = 1; i < MAX_PLAYERS; i++ )
	{
		GetPlayerInfo( i, &m_PlayerInfoList[i] );

		if ( m_PlayerInfoList[i].thisplayer )
			m_iPlayerNum = i;  // !!!HACK: this should be initialized elsewhere... maybe gotten from the engine
	}
}

int CHudScoreboard :: MsgFunc_ScoreInfo( const char *pszName, int iSize, void *pbuf )
{
	m_iFlags |= HUD_ACTIVE;

	BEGIN_READ( pbuf, iSize );
	short cl = READ_BYTE();
	short frags = READ_SHORT();
	short deaths = READ_SHORT();

	if ( cl > 0 && cl <= MAX_PLAYERS )
	{
		m_PlayerExtraInfo[cl].frags = frags;
		m_PlayerExtraInfo[cl].deaths = deaths;
	}

	return 1;
}

int CHudScoreboard :: MsgFunc_JInJail( const char *pszName, int iSize, void *pbuf )
{
	m_iFlags |= HUD_ACTIVE;

	BEGIN_READ( pbuf, iSize );
	short cl = READ_BYTE();
	short flag = READ_BYTE ();

	if ( cl > 0 && cl <= MAX_PLAYERS )
	{
		m_PlayerExtraInfo[cl].injail = flag;
	}

	return 1;
}

int CHudScoreboard :: MsgFunc_JInArmory( const char *pszName, int iSize, void *pbuf )
{
	m_iFlags |= HUD_ACTIVE;

	BEGIN_READ( pbuf, iSize );
	short cl = READ_BYTE();
	short flag = READ_BYTE ();

	if ( cl > 0 && cl <= MAX_PLAYERS )
	{
		m_PlayerExtraInfo[cl].inarmory = flag;
	}

	return 1;
}

// Message handler for TeamInfo message
// accepts two values:
//		byte: client number
//		byte: team number
int CHudScoreboard :: MsgFunc_JTeamInfo( const char *pszName, int iSize, void *pbuf )
{
	BEGIN_READ( pbuf, iSize );
	short cl = READ_BYTE();
	short team = READ_BYTE ();

	//
	// Adjust the team number
	//

	if (team < 0 || team > 0x7F)  // unsigned byte :)
	{
		team = m_iNumTeams;
	}
	else
	{
		team++;
	}
	
	if ( cl > 0 && cl <= MAX_PLAYERS )
	{  // set the players team
		m_PlayerExtraInfo [cl] .teamindex = team;
	}

	// rebuild the list of teams

	// clear out player counts from teams
	for ( int i = 1; i <= m_iNumTeams; i++ )
	{
		m_TeamInfo [i].players = 0;
		m_TeamInfo [i].ownteam = FALSE;
	}

	// rebuild the team list
	GetAllPlayersInfo();
	for ( i = 1; i < MAX_PLAYERS; i++ )
	{
		if ( m_PlayerInfoList[i].name == NULL )
			continue;

		if ( m_PlayerExtraInfo[i].teamindex == 0 )
			continue; // skip over players who are not in a team

		m_TeamInfo[m_PlayerExtraInfo[i].teamindex].players++;

		if ( m_PlayerInfoList[i].thisplayer )
			m_TeamInfo[m_PlayerExtraInfo[i].teamindex].ownteam = TRUE;
	}

	return 1;
}

// Message handler for TeamScore message
// accepts three values:
//		byte: team index
//		short: teams executions
// if this message is never received, then scores will simply be the combined totals of the players.
int CHudScoreboard :: MsgFunc_JTeamScore( const char *pszName, int iSize, void *pbuf )
{
	BEGIN_READ( pbuf, iSize );

	short team = READ_BYTE ();
	if (team < 0 || team > 0x7F)  // unsigned byte :)
	{
		team = m_iNumTeams;
	}
	else
	{
		team++;
	}

	// use this new score data instead of combined player scores
	m_TeamInfo[team].scores_overriden = TRUE;
	m_TeamInfo[team].executions = READ_SHORT();
	m_fTeamScore = TRUE;	
	return 1;
}

// Message handler for JTeamList message
// accepts 2 values:
//		byte: number of teams
//		string: list of team names
// if this message is never received, then scores will simply be the combined totals of the players.
int CHudScoreboard :: MsgFunc_JTeamList( const char *pszName, int iSize, void *pbuf )
{
	BEGIN_READ( pbuf, iSize );

	m_iNumTeams = READ_BYTE ();

	for (int i = 1; i <= m_iNumTeams; i++)
	{
		const char *pTeamName = READ_STRING ();
		strncpy( m_TeamInfo[i].name, pTeamName, MAX_TEAM_NAME );
	}
	
	return 1;
}

void CHudScoreboard :: DeathMsg( int killer, int victim )
{
	// if we were the one killed,  or the world killed us, set the scoreboard to indicate suicide
	if ( victim == m_iPlayerNum || killer == 0 )
	{
		m_iLastKilledBy = killer ? killer : m_iPlayerNum;
		m_fLastKillTime = gHUD.m_flTime + 10;	// display who we were killed by for 10 seconds

		if ( killer == m_iPlayerNum )
			m_iLastKilledBy = m_iPlayerNum;
	}
}



void CHudScoreboard :: UserCmd_ShowScores( void )
{
	m_iShowscoresHeld = TRUE;
}

void CHudScoreboard :: UserCmd_HideScores( void )
{
	m_iShowscoresHeld = FALSE;
}
