/*------------------------------------------------------------------------------
*
*	Half-Life JailBreak Revision Log
*
*	$History: jailbreak_gamerules.h $
*       
*       *****************  Version 17  *****************
*       User: Tim Smith    Date: 5/01/99    Time: 3:09p
*       Updated in $/HLJB/dlls
*       Modified game description
*       
*       *****************  Version 16  *****************
*       User: Tim Smith    Date: 4/27/99    Time: 10:08a
*       Updated in $/HLJB/dlls
*       Added team based scoring system
*       
*       *****************  Version 15  *****************
*       User: Tim Smith    Date: 4/26/99    Time: 1:11p
*       Updated in $/HLJB/dlls
*       1) When the initial team selection menu is displayed, the observer is
*       frozen
*       2) Observers will try to spawn at info_intermission entities first
*       
*       *****************  Version 14  *****************
*       User: Tim Smith    Date: 4/23/99    Time: 10:26a
*       Updated in $/HLJB/dlls
*       Added server config variables to the voting system and cleaned up some
*       code.
*       
*       *****************  Version 13  *****************
*       User: Tim Smith    Date: 4/22/99    Time: 9:48p
*       Updated in $/HLJB/dlls
*       Initial version of voting system completed
*       
*       *****************  Version 12  *****************
*       User: Tim Smith    Date: 4/22/99    Time: 11:33a
*       Updated in $/HLJB/dlls
*       Added the players command
*       
*       *****************  Version 11  *****************
*       User: Tim Smith    Date: 4/21/99    Time: 10:11a
*       Updated in $/HLJB/dlls
*       1) Players who exit should no longer be counted as members of a team.
*       The free flag on the entity was not being set by the engine.  The team
*       value is not being cleared.
*       2) Players will now start off the with 9mm. (That one is for you Haak)
*       3) Players will teleport after an execution of the other team.  Not be
*       killed.
*       
*       *****************  Version 10  *****************
*       User: Tim Smith    Date: 4/20/99    Time: 12:56p
*       Updated in $/HLJB/dlls
*       1) Team colors are now working.
*       2) Model names have been changed from grunt/recon to red/blue
*       
*       *****************  Version 9  *****************
*       User: Tim Smith    Date: 4/16/99    Time: 11:59a
*       Updated in $/HLJB/dlls
*       Added the commands RADIO# for the 9 precanned messages.
*       
*       *****************  Version 8  *****************
*       User: Tim Smith    Date: 4/15/99    Time: 3:34p
*       Updated in $/HLJB/dlls
*       Totally removed the subclassing of teamplay_gamerules.
*       jailbreak_gamerules is now a subclass of multiplay_gamerules.
*       
*       *****************  Version 7  *****************
*       User: Tim Smith    Date: 4/15/99    Time: 12:56p
*       Updated in $/HLJB/dlls
*       1) Added player id.
*       2) Corrected bug that allowed you to move around after being executed
*       3) Corrected bug that prevented forcerespawn if a button was held
*       
*       *****************  Version 6  *****************
*       User: Tim Smith    Date: 4/14/99    Time: 4:12p
*       Updated in $/HLJB/dlls
*       Improved the amount of time it takes for an observer to respawn.  Also
*       reduced the amount of time it takes for a player to auto-respawn.  (was
*       6, now 3 + death animation time)
*       
*       *****************  Version 5  *****************
*       User: Tim Smith    Date: 4/14/99    Time: 10:52a
*       Updated in $/HLJB/dlls
*       Modified the team assignments to not use the string, but use the team
*       enum.
*       Added code to set the color of the teams, but there is a problem.
*       
*       *****************  Version 4  *****************
*       User: Tim Smith    Date: 4/13/99    Time: 7:50p
*       Updated in $/HLJB/dlls
*       Completed basic jail functions including custom executions.
*       Minor modifications to the radio system.
*       
*       *****************  Version 3  *****************
*       User: Tim Smith    Date: 4/11/99    Time: 3:57p
*       Updated in $/HLJB/dlls
*       Radio system has been added.  This includes flood protection.
*       
*       *****************  Version 2  *****************
*       User: Tim Smith    Date: 4/11/99    Time: 11:24a
*       Updated in $/HLJB/dlls
*       Completed initial implementation of the team selection system
*       
*       *****************  Version 1  *****************
*       User: Tim Smith    Date: 4/10/99    Time: 8:41a
*       Created in $/HLJB/dlls
*       
*-----------------------------------------------------------------------------*/

//
// jailbreak_gamerules.h
//

#define MAX_TEAMNAME_LENGTH	16
#define MAX_TEAMS			32

#define TEAMPLAY_TEAMLISTLENGTH		MAX_TEAMS*MAX_TEAMNAME_LENGTH

class CJailBreakTeamplay : public CHalfLifeMultiplay
{
public:
	CJailBreakTeamplay();

	virtual BOOL ClientCommand( CBasePlayer *pPlayer, const char *pcmd );
	virtual void ClientUserInfoChanged( CBasePlayer *pPlayer, char *infobuffer );
	virtual BOOL IsTeamplay (void);
	virtual BOOL FPlayerCanTakeDamage( CBasePlayer *pPlayer, CBaseEntity *pAttacker );
	virtual int PlayerRelationship( CBaseEntity *pPlayer, CBaseEntity *pTarget );
	virtual const char *GetTeamID( CBaseEntity *pEntity );
	virtual BOOL ShouldAutoAim( CBasePlayer *pPlayer, edict_t *target );
	virtual int IPointsForKill( CBasePlayer *pAttacker, CBasePlayer *pKilled );
	virtual void InitHUD( CBasePlayer *pl );		// the client dll is ready for updating
	virtual void DeathNotice( CBasePlayer *pVictim, entvars_t *pKiller, entvars_t *pevInflictor );
	virtual const char *GetGameDescription( void ) { return "JailBreak"; }  // this is the game name that gets seen in the server browser
	virtual void UpdateGameMode( CBasePlayer *pPlayer );  // the client needs to be informed of the current game mode
	virtual void PlayerKilled( CBasePlayer *pVictim, entvars_t *pKiller, entvars_t *pInflictor );
	virtual void Think ( void );
	virtual int GetTeamIndex( const char *pTeamName );
	virtual const char *GetIndexedTeamName( int teamIndex );
	virtual BOOL IsValidTeam( const char *pTeamName );
	virtual const char *SetDefaultPlayerTeam( CBasePlayer *pPlayer );
	virtual void ChangePlayerTeam( CBasePlayer *pPlayer, const char *pTeamName, BOOL bKill, BOOL bGib );
	virtual void PlayerSpawn( CBasePlayer *pPlayer );
	virtual void ClientDisconnected( edict_t *pClient );

	void KillPlayer (CBasePlayer *pPlayer, BOOL bGib);

protected:
	virtual void ChangeLevel( void );

	void RefreshMapList (bool fRemoveDups);
	void DisplaySelectTeamMenu (CBasePlayer *pPlayer, BOOL fNoCancel);
	void DisplayRadioMenu (CBasePlayer *pPlayer);
	void DisplayRadioMessageMenu (CBasePlayer *pPlayer);
	void DisplayVoteMapMenu (CBasePlayer *pPlayer);
	void SayPlay (CBasePlayer *pPlayer, BOOL fRadio, BOOL fTeam, BOOL fClose, 
			const char *pszSound, const char *pszMessage);
	void ExecuteTeam (int nTeam);
	int TeamWithFewestPlayers (CBasePlayer *pSkipPlayer);
	void ParseTeamString( void );
	void RadioMessage (CBasePlayer *pPlayer, int slot);
	void SetPlayerColor (CBasePlayer *pPlayer, int nColor);

protected: //commands
	BOOL Cmd_MenuSelect (CBasePlayer *pPlayer, const char *pcmd);
	BOOL Cmd_ChangeTeam (CBasePlayer *pPlayer, const char *pcmd);
	BOOL Cmd_Say (CBasePlayer *pPlayer, BOOL fTeam, BOOL fNoArg0);
	BOOL Cmd_Radio (CBasePlayer *pPlayer, BOOL fTeam, BOOL fClose);
	BOOL Cmd_RadioSound (CBasePlayer *pPlayer);
	BOOL Cmd_RadioText (CBasePlayer *pPlayer);
	BOOL Cmd_RadioOpponents (CBasePlayer *pPlayer);
	BOOL Cmd_RadioMenu (CBasePlayer *pPlayer);
	BOOL Cmd_ID (CBasePlayer *pPlayer);
	BOOL Cmd_Players (CBasePlayer *pPlayer);
	BOOL Cmd_Vote (CBasePlayer *pPlayer);
	BOOL Cmd_Menu (CBasePlayer *pPlayer);
	BOOL Cmd_Help (CBasePlayer *pPlayer);
	BOOL Cmd_Credits (CBasePlayer *pPlayer);

protected:
	float m_fExecCheckTime;
	BOOL m_DisableDeathMessages;
	BOOL m_DisableDeathPenalty;
	char m_szTeamList[TEAMPLAY_TEAMLISTLENGTH];

	static int gm_iNumTeams;
	static int gm_iTeamScores [MAX_TEAMS];

	//
	// Voting system
	//

	enum _Vote 
	{
		Vote_None			= 0x0,
		Vote_Map			= 0x1,
	};

	_Vote m_nVote;
	float m_fVoteEndTime;
	char m_szVoteText [64];

	void BeginVote (CBasePlayer *pPlayer, _Vote nVote, const char *pszVoteText);
};
